"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApiKeysController = exports.isApiEnabledMiddleware = void 0;
const api_types_1 = require("@n8n/api-types");
const api_key_repository_1 = require("../databases/repositories/api-key.repository");
const decorators_1 = require("../decorators");
const bad_request_error_1 = require("../errors/response-errors/bad-request.error");
const event_service_1 = require("../events/event.service");
const license_1 = require("../license");
const public_api_1 = require("../public-api");
const public_api_key_service_1 = require("../services/public-api-key.service");
const isApiEnabledMiddleware = (_, res, next) => {
    if ((0, public_api_1.isApiEnabled)()) {
        next();
    }
    else {
        res.status(404).end();
    }
};
exports.isApiEnabledMiddleware = isApiEnabledMiddleware;
let ApiKeysController = class ApiKeysController {
    constructor(eventService, publicApiKeyService, apiKeysRepository, license) {
        this.eventService = eventService;
        this.publicApiKeyService = publicApiKeyService;
        this.apiKeysRepository = apiKeysRepository;
        this.license = license;
    }
    async createAPIKey(req, _res, { label, expiresAt }) {
        const currentNumberOfApiKeys = await this.apiKeysRepository.countBy({ userId: req.user.id });
        if (currentNumberOfApiKeys >= this.license.getApiKeysPerUserLimit()) {
            throw new bad_request_error_1.BadRequestError('You have reached the maximum number of API keys allowed.');
        }
        const newApiKey = await this.publicApiKeyService.createPublicApiKeyForUser(req.user, {
            label,
            expiresAt,
        });
        this.eventService.emit('public-api-key-created', { user: req.user, publicApi: false });
        return {
            ...newApiKey,
            apiKey: this.publicApiKeyService.redactApiKey(newApiKey.apiKey),
            rawApiKey: newApiKey.apiKey,
            expiresAt,
        };
    }
    async getAPIKeys(req) {
        const apiKeys = await this.publicApiKeyService.getRedactedApiKeysForUser(req.user);
        return apiKeys;
    }
    async deleteAPIKey(req, _res, apiKeyId) {
        await this.publicApiKeyService.deleteApiKeyForUser(req.user, apiKeyId);
        this.eventService.emit('public-api-key-deleted', { user: req.user, publicApi: false });
        return { success: true };
    }
    async updateAPIKey(req, _res, apiKeyId, { label }) {
        await this.publicApiKeyService.updateApiKeyForUser(req.user, apiKeyId, {
            label,
        });
        return { success: true };
    }
};
exports.ApiKeysController = ApiKeysController;
__decorate([
    (0, decorators_1.Post)('/', { middlewares: [exports.isApiEnabledMiddleware] }),
    __param(2, decorators_1.Body),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Response,
        api_types_1.CreateApiKeyRequestDto]),
    __metadata("design:returntype", Promise)
], ApiKeysController.prototype, "createAPIKey", null);
__decorate([
    (0, decorators_1.Get)('/', { middlewares: [exports.isApiEnabledMiddleware] }),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object]),
    __metadata("design:returntype", Promise)
], ApiKeysController.prototype, "getAPIKeys", null);
__decorate([
    (0, decorators_1.Delete)('/:id', { middlewares: [exports.isApiEnabledMiddleware] }),
    __param(2, (0, decorators_1.Param)('id')),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Response, String]),
    __metadata("design:returntype", Promise)
], ApiKeysController.prototype, "deleteAPIKey", null);
__decorate([
    (0, decorators_1.Patch)('/:id', { middlewares: [exports.isApiEnabledMiddleware] }),
    __param(2, (0, decorators_1.Param)('id')),
    __param(3, decorators_1.Body),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", [Object, Response, String, api_types_1.UpdateApiKeyRequestDto]),
    __metadata("design:returntype", Promise)
], ApiKeysController.prototype, "updateAPIKey", null);
exports.ApiKeysController = ApiKeysController = __decorate([
    (0, decorators_1.RestController)('/api-keys'),
    __metadata("design:paramtypes", [event_service_1.EventService,
        public_api_key_service_1.PublicApiKeyService,
        api_key_repository_1.ApiKeyRepository,
        license_1.License])
], ApiKeysController);
//# sourceMappingURL=api-keys.controller.js.map