import { CreateProjectDto, DeleteProjectDto, UpdateProjectDto } from '@n8n/api-types';
import type { Scope } from '@n8n/permissions';
import { Response } from 'express';
import type { Project } from '../databases/entities/project';
import { ProjectRepository } from '../databases/repositories/project.repository';
import { EventService } from '../events/event.service';
import type { ProjectRequest } from '../requests';
import { AuthenticatedRequest } from '../requests';
import { ProjectService } from '../services/project.service.ee';
import { RoleService } from '../services/role.service';
export declare class ProjectController {
    private readonly projectsService;
    private readonly roleService;
    private readonly projectRepository;
    private readonly eventService;
    constructor(projectsService: ProjectService, roleService: RoleService, projectRepository: ProjectRepository, eventService: EventService);
    getAllProjects(req: AuthenticatedRequest): Promise<Project[]>;
    getProjectCounts(): Promise<Record<"personal" | "team", number>>;
    createProject(req: AuthenticatedRequest, _res: Response, payload: CreateProjectDto): Promise<{
        role: string;
        scopes: Scope[];
        name: string;
        type: import("@n8n/api-types").ProjectType;
        icon: import("@n8n/api-types").ProjectIcon;
        projectRelations: import("../databases/entities/project-relation").ProjectRelation[];
        sharedCredentials: import("../databases/entities/shared-credentials").SharedCredentials[];
        sharedWorkflows: import("../databases/entities/shared-workflow").SharedWorkflow[];
        id: string;
        createdAt: Date;
        updatedAt: Date;
    }>;
    getMyProjects(req: AuthenticatedRequest, _res: Response): Promise<ProjectRequest.GetMyProjectsResponse>;
    getPersonalProject(req: AuthenticatedRequest): Promise<{
        scopes: Scope[];
        name: string;
        type: import("@n8n/api-types").ProjectType;
        icon: import("@n8n/api-types").ProjectIcon;
        projectRelations: import("../databases/entities/project-relation").ProjectRelation[];
        sharedCredentials: import("../databases/entities/shared-credentials").SharedCredentials[];
        sharedWorkflows: import("../databases/entities/shared-workflow").SharedWorkflow[];
        id: string;
        createdAt: Date;
        updatedAt: Date;
    }>;
    getProject(req: AuthenticatedRequest, _res: Response, projectId: string): Promise<ProjectRequest.ProjectWithRelations>;
    updateProject(req: AuthenticatedRequest, _res: Response, payload: UpdateProjectDto, projectId: string): Promise<void>;
    deleteProject(req: AuthenticatedRequest, _res: Response, query: DeleteProjectDto, projectId: string): Promise<void>;
}
