import type { PushMessage } from '@n8n/api-types';
import type { Application } from 'express';
import type { Server } from 'http';
import { InstanceSettings, Logger } from 'n8n-core';
import type { User } from '../databases/entities/user';
import { Publisher } from '../scaling/pubsub/publisher.service';
import { TypedEmitter } from '../typed-emitter';
import { SSEPush } from './sse.push';
import type { OnPushMessage, PushResponse, SSEPushRequest, WebSocketPushRequest } from './types';
import { WebSocketPush } from './websocket.push';
type PushEvents = {
    editorUiConnected: string;
    message: OnPushMessage;
};
export declare class Push extends TypedEmitter<PushEvents> {
    private readonly instanceSettings;
    private readonly publisher;
    private readonly logger;
    isBidirectional: boolean;
    private backend;
    constructor(instanceSettings: InstanceSettings, publisher: Publisher, logger: Logger);
    getBackend(): SSEPush | WebSocketPush;
    handleRequest(req: SSEPushRequest | WebSocketPushRequest, res: PushResponse): void;
    broadcast(pushMsg: PushMessage): void;
    hasPushRef(pushRef: string): boolean;
    send(pushMsg: PushMessage, pushRef: string): void;
    sendToUsers(pushMsg: PushMessage, userIds: Array<User['id']>): void;
    onShutdown(): void;
    private shouldRelayViaPubSub;
    private relayViaPubSub;
}
export declare const setupPushServer: (restEndpoint: string, server: Server, app: Application) => void;
export declare const setupPushHandler: (restEndpoint: string, app: Application) => void;
export {};
