"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.objectToError = objectToError;
const n8n_core_1 = require("n8n-core");
const n8n_workflow_1 = require("n8n-workflow");
function objectToError(errorObject, workflow) {
    if (errorObject instanceof Error) {
        return errorObject;
    }
    else if ((0, n8n_core_1.isObjectLiteral)(errorObject) &&
        'message' in errorObject &&
        typeof errorObject.message === 'string') {
        let error;
        if ('node' in errorObject &&
            (0, n8n_core_1.isObjectLiteral)(errorObject.node) &&
            typeof errorObject.node.name === 'string') {
            const node = workflow.getNode(errorObject.node.name);
            if (node) {
                error = new n8n_workflow_1.NodeOperationError(node, errorObject, errorObject);
            }
        }
        if (error === undefined) {
            error = new Error(errorObject.message);
        }
        if ('description' in errorObject) {
            error.description = errorObject.description;
        }
        if ('stack' in errorObject) {
            error.stack = errorObject.stack;
        }
        return error;
    }
    else {
        return new Error('An error occurred');
    }
}
//# sourceMappingURL=object-to-error.js.map