import { ErrorReporter, InstanceSettings, Logger } from 'n8n-core';
import type { ExecutionError, IDeferredPromise, IExecuteResponsePromiseData, WorkflowExecuteMode, WorkflowHooks, IWorkflowExecutionDataProcess } from 'n8n-workflow';
import { ActiveExecutions } from './active-executions';
import { ExecutionRepository } from './databases/repositories/execution.repository';
import { ExecutionNotFoundError } from './errors/execution-not-found-error';
import { ManualExecutionService } from './manual-execution.service';
import { NodeTypes } from './node-types';
import { PermissionChecker } from './user-management/permission-checker';
import { WorkflowStaticDataService } from './workflows/workflow-static-data.service';
export declare class WorkflowRunner {
    private readonly logger;
    private readonly errorReporter;
    private readonly activeExecutions;
    private readonly executionRepository;
    private readonly workflowStaticDataService;
    private readonly nodeTypes;
    private readonly permissionChecker;
    private readonly instanceSettings;
    private readonly manualExecutionService;
    private scalingService;
    private executionsMode;
    constructor(logger: Logger, errorReporter: ErrorReporter, activeExecutions: ActiveExecutions, executionRepository: ExecutionRepository, workflowStaticDataService: WorkflowStaticDataService, nodeTypes: NodeTypes, permissionChecker: PermissionChecker, instanceSettings: InstanceSettings, manualExecutionService: ManualExecutionService);
    processError(error: ExecutionError | ExecutionNotFoundError, startedAt: Date, executionMode: WorkflowExecuteMode, executionId: string, hooks?: WorkflowHooks): Promise<void>;
    run(data: IWorkflowExecutionDataProcess, loadStaticData?: boolean, realtime?: boolean, restartExecutionId?: string, responsePromise?: IDeferredPromise<IExecuteResponsePromiseData>): Promise<string>;
    private runMainProcess;
    private enqueueExecution;
}
